#pragma once
#define GIT_COMMIT_HASH    "efb50993780079460b0cbed1363e2166a2de1d9f"
#define GIT_BRANCH         "v0.56.2"
#define GIT_COMMIT_MESSAGE "[gha] Nix: update inputs"
#define GIT_COMMIT_DATE    "Wed Aug 5 14:13:21 2026"
#define GIT_DIRTY          "clean"
#define GIT_TAG            "v0.56.2"
#define GIT_COMMITS        "7661"

#define AQUAMARINE_VERSION "0.15.0"
// clang-format off
#define AQUAMARINE_VERSION_MAJOR 0
#define AQUAMARINE_VERSION_MINOR 15
#define AQUAMARINE_VERSION_PATCH 0
// clang-format on
#define HYPRLANG_VERSION     "0.6.8"
#define HYPRUTILS_VERSION    "0.14.1"
#define HYPRCURSOR_VERSION   "0.1.13"
#define HYPRGRAPHICS_VERSION "0.5.1"
